from sqlalchemy import Column, String
from sqlalchemy.orm import declarative_base

Base = declarative_base()


class Dfkkzp(Base):
    __tablename__ = 'DFKKZP'

    mandt = Column(String, primary_key=True)
    keyz1 = Column(String)
    posza = Column(String)
    selt1 = Column(String)
    selt2 = Column(String)
    selt3 = Column(String)
    selw1 = Column(String)
    selw2 = Column(String)
    selw3 = Column(String)
    betrz = Column(String)
    betrh = Column(String)
    betrk = Column(String)
    betrl = Column(String)
    xtkla = Column(String)
    tbetr = Column(String)
    fikey = Column(String)
    bvrko = Column(String)
    bukrs = Column(String)
    gsber = Column(String)
    blart = Column(String)
    waers = Column(String)
    kursf = Column(String)
    budat = Column(String)
    bldat = Column(String)
    valut = Column(String)
    xeiph = Column(String)
    augrd = Column(String)
    aenam = Column(String)
    aedat = Column(String)
    aetim = Column(String)
    xrues = Column(String)
    xakon = Column(String)
    xklae = Column(String)
    klaeb = Column(String)
    klaeh = Column(String)
    klaed = Column(String)
    opbel = Column(String)
    uposz = Column(String)
    uposv = Column(String)
    txtvw = Column(String)
    chckn = Column(String)
    banks = Column(String)
    bankl = Column(String)
    bankn = Column(String)
    bkont = Column(String)
    koinh = Column(String)
    xpgro = Column(String)
    xdaub = Column(String)
    nrzaa = Column(String)
    repym = Column(String)
    xclar = Column(String)
    infof = Column(String)
    kukon = Column(String)
    ccins = Column(String)
    ccnum = Column(String)
    ruebl = Column(String)
    ruear = Column(String)
    bkref = Column(String)
    txtrz = Column(String)
    hzuon = Column(String)
    datab = Column(String)
    datbi = Column(String)
    aunum = Column(String)
    audat = Column(String)
    autim = Column(String)
    klaec = Column(String)
    xatts = Column(String)
    cguid = Column(String)
    enctp = Column(String)
    merch = Column(String)


class Dfkkzk(Base):
    __tablename__ = 'DFKKZK'

    mandt = Column(String, primary_key=True)
    keyz1 = Column(String)
    keyz2 = Column(String)
    applk = Column(String)
    herkf = Column(String)
    fikey = Column(String)
    bvrko = Column(String)
    bukrs = Column(String)
    gsber = Column(String)
    blart = Column(String)
    waers = Column(String)
    kursf = Column(String)
    budat = Column(String)
    bldat = Column(String)
    valut = Column(String)
    xeiph = Column(String)
    augrd = Column(String)
    selt1 = Column(String)
    selt2 = Column(String)
    selt3 = Column(String)
    anzpo = Column(String)
    summs = Column(String)
    summh = Column(String)
    sumwa = Column(String)
    ernam = Column(String)
    erdat = Column(String)
    ertim = Column(String)
    aenam = Column(String)
    aedat = Column(String)
    aetim = Column(String)
    maxup = Column(String)
    maxuv = Column(String)
    stazs = Column(String)
    xschs = Column(String)
    xcrds = Column(String)
    xzaus = Column(String)
    cczah = Column(String)
    subel = Column(String)
    vkoch = Column(String)
    xebok = Column(String)
    xposa = Column(String)
    infof = Column(String)
    ktsus = Column(String)
    ktsuh = Column(String)
    ksump = Column(String)
    xnseb = Column(String)
    jobname = Column(String)
    chdsk = Column(String)
    paytp = Column(String)
    xauts = Column(String)
    cvscd = Column(String)
    xexpy = Column(String)

class Dfkkopk(Base):
    __tablename__ = 'DFKKOPK'

    mandt = Column(String, primary_key=True)
    opbel = Column(String)
    valut = Column(String)
    fikey = Column(String)
    hkont = Column(String)
    betrh = Column(String)
    intbu = Column(String)
    gsber = Column(String)


class Dfkkopkc(Base):
    __tablename__ = 'DFKKOPKC'

    mandt = Column(String, primary_key=True)
    opbel = Column(String)
    opupk = Column(String)
    ccins = Column(String)
    ccnum = Column(String)
    ccfol = Column(String)
    datab = Column(String)
    datbi = Column(String)
    ccname = Column(String)
    aunum = Column(String)
    autra = Column(String)
    audat = Column(String)
    autim = Column(String)
    auart = Column(String)
    autwr = Column(String)
    ccwae = Column(String)
    merch = Column(String)
    budat = Column(String)
    betrw = Column(String)
    waers = Column(String)
    bukrs = Column(String)
    xsett = Column(String)
    laufi = Column(String)
    stdoc = Column(String)
    laufd_pyp = Column(String)
    laufi_pyp = Column(String)
    orign_pyp = Column(String)
    anzrate = Column(String)
    cvval = Column(String)
    cvvct = Column(String)
    cvvst = Column(String)
    zzacuenta = Column(String)
    valut = Column(String)
    cguid = Column(String)
    enctp = Column(String)
    resend = Column(String)
    xguid = Column(String)


class Dfkksum(Base):
    __tablename__ = 'DFKKSUM'

    mandt = Column(String, primary_key=True)
    fikey = Column(String)
    shkzg = Column(String)
    hkont = Column(String)
    gsber = Column(String)
    budat = Column(String)
    bldat = Column(String)
    valut = Column(String)
    betrh = Column(String)


class Dfkkko(Base):
    __tablename__ = 'DFKKKO'

    mandt = Column(String, primary_key=True)
    opbel = Column(String)
    blart = Column(String)
    budat = Column(String)
    bldat = Column(String)
    storb = Column(String)
    ernam = Column(String)
    cpudt = Column(String)
    cputm = Column(String)
    fikey = Column(String)


class Dfkkop(Base):
    __tablename__ = 'DFKKOP'

    mandt = Column(String, primary_key=True)
    opbel = Column(String)
    gsber = Column(String)
    augst = Column(String)
    vkont = Column(String)
    faedn = Column(String)
    bldat = Column(String)
    betrh = Column(String)
    augbl = Column(String)
    augdt = Column(String)
    augbt = Column(String)
    augob = Column(String)
    blart = Column(String)




import logging
import os
from functools import cache

import paramiko
import pysftp

from ..config import settings

log = logging.getLogger(__name__)


@cache
class Sftp(object):
    # Accept any host key (still wrong see below)
    cnopts = pysftp.CnOpts()
    cnopts.hostkeys = None

    def __init__(self):
        """Constructor Method"""
        # Set connection object to None (initial value)
        self.sftp_client = None
        self.ssh_client = None
        self.hostname = settings.SFTP_HOSTNAME
        self.username = settings.SFTP_USERNAME
        self.password = settings.SFTP_PASSWORD
        self.port = settings.SFTP_PORT

    def connect(self):
        """Connects to the sftp server and returns the sftp connection object"""

        try:
            # Get the sftp connection object
            # create SSH client and connect to sftp host
            self.ssh_client = paramiko.SSHClient()
            self.ssh_client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
            self.ssh_client.connect(
                self.hostname,
                username=self.username,
                password=self.password,
                port=self.port,
                allow_agent=True
            )

            self.sftp_client = self.ssh_client.open_sftp()

        except pysftp.HostKeysException as e:
            self.sftp_client._init_error = True
            # raise paramiko.ssh_exception.SSHException(str(e))
        except Exception as err:
            log.exception(err)
            # raise Exception(err)
        finally:
            log.info(f"Connected to {self.hostname} as {self.username}.")

    def disconnect(self):
        """Closes the sftp connection"""
        self.sftp_client.close()
        log.info(f"Disconnected from host {self.hostname}")

    def listdir(self, remote_path):
        list_files = []
        """lists all the files and directories in the specified path and returns them"""
        for obj in self.sftp_client.listdir(remote_path):
            list_files.append(obj)
        return list_files

    def listdir_attr(self, remote_path):
        """lists all the files and directories (with their attributes) in the specified path and returns them"""
        for attr in self.sftp_client.listdir_attr(remote_path):
            yield attr

    def download(self, remote_path, target_local_path):
        """
        Downloads the file from remote sftp server to local.
        Also, by default extracts the file to the specified target_local_path
        """

        try:
            log.info(
                f"downloading from {self.hostname} as {self.username} [(remote path : {remote_path});(local path: {target_local_path})]"
            )

            # Create the target directory if it does not exist
            path, _ = os.path.split(target_local_path)
            if not os.path.isdir(path):
                try:
                    os.makedirs(path)
                except Exception as err:
                    log.exception(err)
                    raise Exception(err)

            # Download from remote sftp server to local
            self.sftp_client.get(remote_path, target_local_path)
            log.info("download completed")

        except Exception as err:
            log.exception(err)
            raise Exception(err)

    def upload(self, source_local_path, remote_path):
        """
        Uploads the source files from local to the sftp server.
        """

        try:
            log.info(
                f"uploading to {self.hostname} as {self.username} [(remote path: {remote_path});(source local path: {source_local_path})]"
            )

            # Download file from SFTP
            self.sftp_client.put(source_local_path, remote_path)
            log.info("Upload completed")

        except Exception as err:
            log.exception(err)
            raise Exception(err)

    def read_content(self, my_filename):
        with self.sftp_client.open(my_filename) as sftp_file:
            contents = sftp_file.read()
            return contents



import logging

from ..config import settings
from ..utility.utils_messages import UtilsMessages

log = logging.getLogger(__name__)


def petition(json_dict):
    try:
        UtilsMessages.send_messages(settings.KAFKA_TOPIC_PETITION, json_dict)
    except Exception as ex:
        log.exception(ex)



import json
import ssl
import sys

from kafka import KafkaProducer, KafkaConsumer
from kafka.errors import KafkaError

from ..config import constantes


class KafkaConfig:
    producer = None
    consumer = None

    def __get_context(self):
        # Create a new context using system defaults, disable all but TLS1.2
        self.context = ssl.create_default_context()
        self.context.options &= ssl.OP_NO_TLSv1
        self.context.options &= ssl.OP_NO_TLSv1_1

        return self.context

    def __set_producer(self, is_ssl=False):
        if is_ssl:
            self.producer = KafkaProducer(
                bootstrap_servers=[f'{constantes.KAFKA_BOOSTRAP}:{constantes.KAFKA_PORT}'],
                sasl_plain_username=constantes.KAFKA_USERNAME,
                sasl_plain_password=constantes.KAFKA_PASSWORD,
                security_protocol=constantes.KAFKA_SECURITY_PROTOCOL,
                ssl_context=self.__get_context(),
                sasl_mechanism=constantes.KAFKA_SASL_MECHANISM,
                value_serializer=lambda m: json.dumps(m).encode('utf-8'),
                api_version=(0, 10, 1),
                retries=5
            )
        else:
            self.producer = KafkaProducer(
                bootstrap_servers=[f'{constantes.KAFKA_BOOSTRAP}:{constantes.KAFKA_PORT}'],
                value_serializer=lambda m: json.dumps(m).encode('utf-8'),
                api_version=(0, 10, 1),
                retries=5
            )

    def __set_consumer(self, topic, group_id):
        self.consumer = KafkaConsumer(
            topic,
            bootstrap_servers=[constantes.KAFKA_BOOSTRAP],
            api_version=(0, 10, 1),
            auto_offset_reset=constantes.KAFKA_OFFSET,
            max_poll_interval_ms=constantes.KAFKA_TIMEOUT,
            enable_auto_commit=False,
            group_id=group_id
        )

    def send_message(self, topic, message):
        try:
            self.__set_producer()
            future = self.producer.send(topic, message.encode(constantes.KAFKA_ENCODE))
            record_metadata = future.get(timeout=constantes.KAFKA_TIMEOUT)
            self.producer.flush()
        except KafkaError:
            print("Unexpected error:", sys.exc_info()[0])
            raise

    def get_message(self, topic, group_id):
        try:
            return self.__set_consumer(topic, group_id)
        except KafkaError:
            print("Unexpected error:", sys.exc_info()[0])
            raise



import logging

from ..messages.messages import Messages

log = logging.getLogger(__name__)


class UtilsMessages:
    @staticmethod
    def process_messages(topic, group):
        response = []
        msg = Messages()
        try:
            messages = msg.obtiene_mensajes(topico=topic, grupo=group)
            if messages:
                for message in messages:
                    response.append(message)
        except Exception as ex:
            log.exception(ex)

        return response

    @staticmethod
    def send_messages(topic, msg):
        kafka_svc = Messages()
        try:
            kafka_svc.envia_mensaje(topico=topic, mensaje=msg)
            log.info("Mensaje enviado al tópico %s : %s", topic, msg)
        except Exception as ex:
            log.exception(ex)


import logging
import os
import time

import pandas as pd
from bson import ObjectId
from flask import jsonify, make_response
from werkzeug.utils import secure_filename

from ..config import constantes
from ..config import settings
from ..database.mongodb import MongoAPI
from .config_rules import get_file, get_columns_file, get_all_config_rules_by_partner
from .partner import get_partner
from ..utility.utils import get_datetime, allowed_file, get_message_error
from ..utility.utils_file import execute_process_conciliation, convert_file_to_hfs5

log = logging.getLogger(__name__)
n = 10000


def upload_file(request):
    if 'file' not in request.files:
        resp = jsonify({'message': constantes.RESPONSE_NOT_PART_FILE_UPLOAD})
        resp.status_code = 400
        return resp

    file = request.files['file']
    '''if file.st_size == 0:
        resp = jsonify({'message': constantes.RESPONSE_EMPTY_SELECT_FILE_UPLOAD})
        resp.status_code = 400
    el'''
    if file.filename == constantes.EMPTY:
        resp = jsonify({'message': constantes.RESPONSE_NOT_SELECT_FILE_UPLOAD})
        resp.status_code = 400
    elif request.form['type'] is None:
        resp = jsonify({'message': constantes.RESPONSE_NOT_SETTING_TYPE_FILE_UPLOAD})
        resp.status_code = 400
    elif request.form['type'] == constantes.EMPTY:
        resp = jsonify({'message': constantes.RESPONSE_EMPTY_SETTING_TYPE_FILE_UPLOAD})
        resp.status_code = 400
    elif file and allowed_file(file.filename):
        file_name, file_extension = os.path.splitext(secure_filename(file.filename))

        # Save data upload
        type_data = request.form['type']
        partner_id = request.form[f'{settings.MONGO_COLLECTION_UPLOAD_FILE_PARTNER_ID}']
        config_proc = request.form['config_id']
        year = 0
        month = 0

        if 'year' in request.form:
            year = int(request.form['year'])

        if 'month' in request.form:
            month = int(request.form['month'])

        # TODO: Acá debiese colocarse la logica para subir los archivos al SFTP si está configurado así
        if year > 0 and month > 0:
            partner = get_partner(partner_id)
            path_upload = os.path.join(
                settings.FLASK_UPLOAD_FOLDER,
                partner[0]['name'],
                f'{year}',
                f'{month}'
            )

            if not os.path.exists(path_upload):
                os.makedirs(path_upload)

            file.save(os.path.join(path_upload, f'{file_name}{file_extension}'))
            file.close()
            convert_file_to_hfs5(
                os.path.join(partner[0]['name'], f'{year}', f'{month}'),
                file_name,
                file_extension
            )

        response = upload_file_by_partner(
            config_proc,
            type_data,
            file_name,
            file_extension,
            year,
            month
        )
        if response:
            resp = jsonify(response)
            resp.status_code = 201
        else:
            resp = jsonify({'message': constantes.RESPONSE_ERROR_UPLOAD})
            resp.status_code = 500
    else:
        resp = jsonify({'message': f'Las extensiones permitidas son: {settings.FLASK_ALLOWED_EXTENSIONS}'})
        resp.status_code = 400

    return resp


def upload_file_by_partner(
        config_proc: str,
        type_data: str,
        filename: str,
        ext: str,
        year: int,
        month: int):
    response = []
    try:
        time_process = get_datetime()
        upload_file_response = __create_upload_file(time_process, filename, ext, type_data, year, month)
        if upload_file_response:
            upload_file_id = upload_file_response['Document_ID']
            data = {
                'collection': settings.MONGO_COLLECTION_PROC_RECONCILED_FILE,
                'Document': {
                    f'{settings.MONGO_COLLECTION_PROC_RECONCILED_CNF}_id': config_proc,
                    f'{settings.MONGO_COLLECTION_UPLOAD_FILE}_id': upload_file_id,
                    'status': 'not-processed',
                    'created_at': time_process
                },
                'Filter': {
                    f'{settings.MONGO_COLLECTION_PROC_RECONCILED_CNF}_id': config_proc,
                    f'{settings.MONGO_COLLECTION_UPLOAD_FILE}_id': upload_file_id
                }
            }

            mongodb = MongoAPI(data)
            cnf_proc_files = mongodb.write(data)
            if cnf_proc_files:
                response = get_files_upload(config_proc)
    except Exception as e:
        log.exception(e)
        response = get_message_error(e)
    return response


def get_files_upload(config_proc: str):
    data = {
        'collection': settings.MONGO_COLLECTION_PROC_RECONCILED_FILE,
        'Filter': {
            f'{settings.MONGO_COLLECTION_PROC_RECONCILED_CNF}_id': config_proc
        }
    }
    mongodb = MongoAPI(data)
    proc_files = mongodb.read()
    files = []
    for proc_file in proc_files:
        data = {
            'collection': settings.MONGO_COLLECTION_UPLOAD_FILE,
            'Filter': {
                f'_id': ObjectId(proc_file[f'{settings.MONGO_COLLECTION_UPLOAD_FILE}_id'])
            }
        }
        mongodb = MongoAPI(data)
        up_file = mongodb.read()
        if up_file:
            files.append({
                'filename': f'{up_file[0]["filename"]}{up_file[0]["extension"]}',
                'type': up_file[0]["type"]
            })
    return {
        'files': files
    }


def get_index_file(pid: str, type_data: str):
    try:
        data = {
            'collection': settings.MONGO_COLLECTION_CONFIG_RULES_DEFINED_INDEX_COLUMNS,
            'Filter': {
                f'{settings.MONGO_COLLECTION_CONFIG_RULES}_id': pid,
                'type_data': type_data
            }
        }
        mongodb = MongoAPI(data)
        response = mongodb.read()
    except Exception as e:
        log.exception(e)
        response = get_message_error(e)
    return response


def get_columns_amount(pid: str):
    try:
        data = {
            'collection': settings.MONGO_COLLECTION_CONFIG_RULES_DEFINED_AMOUNT_COLUMN,
            'Filter': {
                f'{settings.MONGO_COLLECTION_CONFIG_RULES}_id': pid
            }
        }
        mongodb = MongoAPI(data)
        response = mongodb.read()
    except Exception as e:
        log.exception(e)
        response = get_message_error(e)
    return response


def get_columns_date(pid: str):
    try:
        data = {
            'collection': settings.MONGO_COLLECTION_CONFIG_RULES_DEFINED_DATE_COLUMN,
            'Filter': {
                f'{settings.MONGO_COLLECTION_CONFIG_RULES}_id': pid
            }
        }
        mongodb = MongoAPI(data)
        response = mongodb.read()
    except Exception as e:
        log.exception(e)
        response = get_message_error(e)
    return response


def process_files(json_data: dict):
    """data = {
        'collection': settings.MONGO_COLLECTION_PROC_RECONCILED_CNF,
        'Filter': {
            '_id': ObjectId(json_data.get('doc_id'))
        }
    }
    mongodb = MongoAPI(data)
    response = mongodb.read()
    if len(response) > 0:
        queue_cxp = QueueCxP()
        queue_cxp.add_message(json_data)

    return [{
        'message': 'Proceso lanzado exitosamente'
    }]


def exec_process(json_data: dict):
    while True:"""
    start = time.time()
    date_start = get_datetime()
    process = []
    try:
        data = {
            'collection': settings.MONGO_COLLECTION_PROC_RECONCILED_CNF,
            'Filter': {
                '_id': ObjectId(json_data.get('doc_id'))
            }
        }
        mongodb = MongoAPI(data)
        response = mongodb.read()
        if len(response) > 0:
            config_process = response[0]

            data = {
                'collection': settings.MONGO_COLLECTION_PROC_RECONCILED_FILE,
                'Filter': {
                    f'{settings.MONGO_COLLECTION_PROC_RECONCILED_CNF}_id': config_process['_id']
                }
            }
            mongodb = MongoAPI(data)
            config_process_files = mongodb.read()

            documents = []
            columns = []
            filters = []
            amounts = []
            dates = []
            for cnf_proc_file in config_process_files:
                # Obtiene detalle de Archivo asociado a ID
                data_file = get_file(cnf_proc_file[f'{settings.MONGO_COLLECTION_UPLOAD_FILE}_id'])
                if len(data_file) > 0:
                    up_file = data_file[0]
                    documents.append(up_file)

                    # Obtiene detalle de las columnas configuradas por archivo
                    cnf_rules_columns = get_columns_file(
                        config_process[f'{settings.MONGO_COLLECTION_CONFIG_RULES}_id'],
                        up_file['type']
                    )
                    if len(cnf_rules_columns) > 0:
                        columns.append(cnf_rules_columns['columns'])

                    # Obtiene el indice a utilizar
                    cnf_rules_indexes = get_index_file(
                        config_process[f'{settings.MONGO_COLLECTION_CONFIG_RULES}_id'],
                        up_file['type']
                    )
                    if len(cnf_rules_indexes) > 0:
                        filters.append(cnf_rules_indexes[0]['columns'])

                    # Obtiene la columna totalizadora a utilizar si existe
                    cnf_amount_columns = get_columns_amount(
                        config_process[f'{settings.MONGO_COLLECTION_CONFIG_RULES}_id']
                    )
                    if len(cnf_amount_columns) > 0 and cnf_amount_columns[0]['column'] not in amounts:
                        amounts.append(cnf_amount_columns[0]['column'])

                    # Obtiene la columna fecha de filtro a utilizar si existe
                    cnf_date_columns = get_columns_date(
                        config_process[f'{settings.MONGO_COLLECTION_CONFIG_RULES}_id']
                    )
                    if len(cnf_date_columns) > 0 and cnf_date_columns[0]['column'] not in dates:
                        dates.append(cnf_date_columns[0]['column'])

            if len(documents) > 0 and len(columns) > 0 and len(filters) > 0:
                external = []
                columns_df = []
                columns_idx = []
                columns_normalized = {}
                for col in columns:
                    for c in col:
                        if c.get('col_used') and c.get('col_name') not in columns_df:
                            columns_df.append(c.get('col_name'))

                        if c.get('col_norm') and c.get('col_used') and c.get('col_name') not in columns_normalized:
                            columns_normalized[c.get('col_name')] = c.get('col_type')

                for col in filters:
                    for c in col:
                        if c.get('col_name') not in columns_idx:
                            columns_idx.append(c.get('col_name'))

                if len(columns_df) > 0:
                    filter_external_data = list(filter(lambda d: (d['type'] == 'primary'), documents))
                    filter_data = filter_external_data[0]

                    partner_name = \
                        get_partner(config_process[f'{settings.MONGO_COLLECTION_UPLOAD_FILE_PARTNER_ID}'])[0]['name']

                    internal = {
                        'year': int(filter_data['year']),
                        'month': int(filter_data['month']),
                        'partner': partner_name,
                        'filename': filter_data['filename'],
                        'extension': filter_data['extension']
                    }

                    filter_external_data = list(filter(lambda d: (d['type'] == 'external'), documents))
                    for filter_data in filter_external_data:
                        external_items = {
                            'year': int(filter_data['year']),
                            'month': int(filter_data['month']),
                            'partner': partner_name,
                            'filename': filter_data['filename'],
                            'extension': filter_data['extension']
                        }
                        external.append(external_items)

                    # Ejecución del proceso conciliación
                    process_detail = {
                        'internal': internal,
                        'external': external,
                        'columns_normalized': columns_normalized,
                        'columns_df': columns_df,
                        'columns_index_df': columns_idx,
                        'columns_amount': amounts,
                        'columns_date': dates
                    }
                    process.append(process_detail)

                    # Inicio Proceso Conciliación
                    result = execute_process_conciliation(process, json_data.get('doc_id'))
                    if result and 'msg' not in result:
                        date_process_executed = get_datetime()
                        # Se almacena resultado de conciliados y no conciliados en MongoDB
                        dict_reconciled_primary = result.get('primary-process').get('data_reconciled')
                        dict_reconciled_secondary = result.get('secondary-process').get('data_reconciled')
                        dict_unreconciled_primary = result.get('primary-process').get('data_unreconciled')
                        dict_unreconciled_secondary = result.get('secondary-process').get('data_unreconciled')
                        dict_duplicated_primary = result.get('primary-process').get('data_duplicated')
                        dict_duplicated_secondary = result.get('secondary-process').get('data_duplicated')

                        if len(dict_reconciled_primary) > 0 or len(dict_reconciled_secondary) > 0:
                            # Se eliminan resultado de datos conciliados
                            data = {
                                'collection': settings.MONGO_COLLECTION_DATA_RECONCILED,
                                'Filter': {
                                    f'{settings.MONGO_COLLECTION_PROC_RECONCILED_CNF}_id': json_data.get('doc_id')
                                }
                            }
                            mongodb = MongoAPI(data)
                            mongodb.delete_many(data)

                            # Se almacena resultado de datos conciliados
                            if len(dict_reconciled_primary) > 0:
                                data['Filter']['type_process'] = 'primary'
                                mongodb = MongoAPI(data)
                                mongodb.write_many(dict_reconciled_primary)

                            if len(dict_reconciled_secondary) > 0:
                                data['Filter']['type_process'] = 'secondary'
                                mongodb = MongoAPI(data)
                                mongodb.write_many(dict_reconciled_secondary)

                            # Se almacena total de conciliados
                            if len(dict_reconciled_primary) > 0:
                                data = {
                                    'collection': settings.MONGO_COLLECTION_DATA_TOTAL_RECONCILED,
                                    'Document': {
                                        'total_reconciled': int(result.get('primary-process').get('total_reconciled')),
                                        'type_process': 'primary',
                                        'date_process': date_process_executed,
                                        f'{settings.MONGO_COLLECTION_PROC_RECONCILED_CNF}_id': json_data.get('doc_id')
                                    },
                                    'Filter': {
                                        f'{settings.MONGO_COLLECTION_PROC_RECONCILED_CNF}_id': json_data.get('doc_id'),
                                        'type_process': 'primary'
                                    }
                                }
                                mongodb = MongoAPI(data)
                                mongodb.write(data)

                            if len(dict_reconciled_secondary) > 0:
                                data = {
                                    'collection': settings.MONGO_COLLECTION_DATA_TOTAL_RECONCILED,
                                    'Document': {
                                        'total_reconciled': int(
                                            result.get('secondary-process').get('total_reconciled')),
                                        'type_process': 'secondary',
                                        'date_process': date_process_executed,
                                        f'{settings.MONGO_COLLECTION_PROC_RECONCILED_CNF}_id': json_data.get('doc_id')
                                    },
                                    'Filter': {
                                        f'{settings.MONGO_COLLECTION_PROC_RECONCILED_CNF}_id': json_data.get('doc_id'),
                                        'type_process': 'secondary'
                                    }
                                }
                                mongodb = MongoAPI(data)
                                mongodb.write(data)

                        if len(dict_unreconciled_primary) > 0 or len(dict_unreconciled_secondary) > 0:
                            # Se eliminan resultado de datos no conciliados
                            data = {
                                'collection': settings.MONGO_COLLECTION_DATA_UNRECONCILED,
                                'Filter': {
                                    f'{settings.MONGO_COLLECTION_PROC_RECONCILED_CNF}_id': json_data.get('doc_id')
                                }
                            }
                            mongodb = MongoAPI(data)
                            mongodb.delete_many(data)

                            # Se almacenan resultados de datos no conciliados
                            if len(dict_unreconciled_primary) > 0:
                                data['Filter']['type_process'] = 'primary'
                                mongodb = MongoAPI(data)
                                mongodb.write_many(dict_unreconciled_primary)

                            if len(dict_unreconciled_secondary) > 0:
                                data['Filter']['type_process'] = 'secondary'
                                mongodb = MongoAPI(data)
                                mongodb.write_many(dict_unreconciled_secondary)

                            # Se almacena total de no conciliados
                            if len(dict_unreconciled_primary) > 0:
                                data = {
                                    'collection': settings.MONGO_COLLECTION_DATA_TOTAL_UNRECONCILED,
                                    'Document': {
                                        'total_unreconciled': int(
                                            result.get('primary-process').get('total_unreconciled')
                                        ),
                                        'type_process': 'primary',
                                        'date_process': date_process_executed,
                                        f'{settings.MONGO_COLLECTION_PROC_RECONCILED_CNF}_id': json_data.get('doc_id')
                                    },
                                    'Filter': {
                                        f'{settings.MONGO_COLLECTION_PROC_RECONCILED_CNF}_id': json_data.get('doc_id'),
                                        'type_process': 'primary'
                                    }
                                }
                                mongodb = MongoAPI(data)
                                mongodb.write(data)

                            if len(dict_unreconciled_secondary) > 0:
                                data = {
                                    'collection': settings.MONGO_COLLECTION_DATA_TOTAL_UNRECONCILED,
                                    'Document': {
                                        'total_unreconciled': int(
                                            result.get('secondary-process').get('total_unreconciled')),
                                        'type_process': 'secondary',
                                        'date_process': date_process_executed,
                                        f'{settings.MONGO_COLLECTION_PROC_RECONCILED_CNF}_id': json_data.get('doc_id')
                                    },
                                    'Filter': {
                                        f'{settings.MONGO_COLLECTION_PROC_RECONCILED_CNF}_id': json_data.get('doc_id'),
                                        'type_process': 'secondary'
                                    }
                                }
                                mongodb = MongoAPI(data)
                                mongodb.write(data)

                        if len(dict_duplicated_primary) > 0 or len(dict_duplicated_secondary) > 0:
                            # Se eliminan resultado de datos duplicados
                            data = {
                                'collection': settings.MONGO_COLLECTION_DATA_DUPLICATES,
                                'Filter': {
                                    f'{settings.MONGO_COLLECTION_PROC_RECONCILED_CNF}_id': json_data.get('doc_id')
                                }
                            }
                            mongodb = MongoAPI(data)
                            mongodb.delete_many(data)

                            # Se almacena resultado de datos duplicados
                            if len(dict_duplicated_primary) > 0:
                                data['Filter']['type_process'] = 'primary'
                                mongodb = MongoAPI(data)
                                mongodb.write_many(dict_duplicated_primary)

                            if len(dict_duplicated_secondary) > 0:
                                data['Filter']['type_process'] = 'secondary'
                                mongodb = MongoAPI(data)
                                mongodb.write_many(dict_duplicated_secondary)

                            # Se almacena total de duplicados

                            if len(dict_duplicated_primary) > 0:
                                data = {
                                    'collection': settings.MONGO_COLLECTION_DATA_TOTAL_DUPLICATES,
                                    'Document': {
                                        'total_duplicated': int(result.get('primary-process').get('total_duplicated')),
                                        'type_process': 'primary',
                                        'date_process': date_process_executed,
                                        f'{settings.MONGO_COLLECTION_PROC_RECONCILED_CNF}_id': json_data.get('doc_id')
                                    },
                                    'Filter': {
                                        f'{settings.MONGO_COLLECTION_PROC_RECONCILED_CNF}_id': json_data.get('doc_id'),
                                        'type_process': 'primary'
                                    }
                                }
                                mongodb = MongoAPI(data)
                                mongodb.write(data)

                            if len(dict_duplicated_secondary) > 0:
                                data = {
                                    'collection': settings.MONGO_COLLECTION_DATA_TOTAL_DUPLICATES,
                                    'Document': {
                                        'total_duplicated': int(result.get('primary-process').get('total_duplicated')),
                                        'type_process': 'secondary',
                                        'date_process': date_process_executed,
                                        f'{settings.MONGO_COLLECTION_PROC_RECONCILED_CNF}_id': json_data.get('doc_id')
                                    },
                                    'Filter': {
                                        f'{settings.MONGO_COLLECTION_PROC_RECONCILED_CNF}_id': json_data.get('doc_id'),
                                        'type_process': 'secondary'
                                    }
                                }
                                mongodb = MongoAPI(data)
                                mongodb.write(data)

                        end = time.time()
                        date_end = get_datetime()

                        # Actualizar estado del proceso en la tabla
                        data = {
                            'collection': settings.MONGO_COLLECTION_PROC_RECONCILED_CNF,
                            'Filter': {
                                '_id': ObjectId(json_data.get('doc_id'))
                            },
                            'DataToBeUpdated': {
                                'state': constantes.CODE_STATE_CONFIG_EXECUTED,
                                'executed_count': int(config_process['executed_count']) + 1,
                                'start_process': date_start,
                                'end_process': date_end,
                                'time_process': round((end - start) / 60, 2),
                                'total_reconciled': int(result.get('primary-process').get('total_reconciled')),
                                'total_unreconciled': int(result.get('primary-process').get('total_unreconciled')),
                                'total_duplicated': int(result.get('primary-process').get('total_duplicated'))
                            }
                        }
                        mongodb = MongoAPI(data)
                        mongodb.update()
                    else:
                        return make_response(jsonify(result), 400)
            response = get_all('-')
    except KeyError as e:
        log.exception(e)
        response = get_message_error(e)
    except Exception as e:
        log.exception(e)
        response = get_message_error(e)

    return response


def get_all(cid: str):
    response = []
    try:
        data = {
            'collection': settings.MONGO_COLLECTION_PROC_RECONCILED_CNF
        }
        if cid != '-':
            data['Filter'] = {
                f'{settings.MONGO_COLLECTION_UPLOAD_FILE_PARTNER_ID}': cid
            }

        mongodb = MongoAPI(data)
        reconciled_cnf = mongodb.read()
        for rec_cnf in reconciled_cnf:
            partner = get_partner(rec_cnf[f'{settings.MONGO_COLLECTION_UPLOAD_FILE_PARTNER_ID}'])
            rule = get_data_rule(rec_cnf[f'{settings.MONGO_COLLECTION_CONFIG_RULES}_id'])
            row = {
                'id': rec_cnf["_id"],
                f'{settings.MONGO_COLLECTION_UPLOAD_FILE_PARTNER_ID}': rec_cnf[
                    f'{settings.MONGO_COLLECTION_UPLOAD_FILE_PARTNER_ID}'
                ],
                'rule_name': rule[0]['name_rule'] if len(rule) > 0 else 'Sin Nombre Regla',
                'partner_name': partner[0]['name'],
                'period': f'{rec_cnf["month"]}/{rec_cnf["year"]}',
                'config_state': rec_cnf['state'],
                'executed_state': __get_state_config_reconciled(rec_cnf['state']),
                'created_at': rec_cnf["created_at"],
                'executed_count': 0 if 'executed_count' not in rec_cnf else int(rec_cnf["executed_count"]),
                'year': int(rec_cnf["year"]),
                'month': int(rec_cnf["month"]),
                'config_id': rec_cnf["config_rules_id"],
                'start_process': '' if 'start_process' not in rec_cnf else rec_cnf["start_process"],
                'end_process': '' if 'end_process' not in rec_cnf else rec_cnf["end_process"],
                'time_process': '' if 'time_process' not in rec_cnf else rec_cnf["time_process"],
                'total_reconciled': 0 if 'total_reconciled' not in rec_cnf else int(rec_cnf['total_reconciled']),
                'total_unreconciled': 0 if 'total_unreconciled' not in rec_cnf else int(rec_cnf['total_unreconciled']),
                'total_duplicated': 0 if 'total_duplicated' not in rec_cnf else int(rec_cnf['total_duplicated'])
            }

            response.append(row)
    except Exception as e:
        log.exception(e)
        response = get_message_error(e)
    return response


def get_data_rule(pid: str):
    try:
        data = {
            'collection': settings.MONGO_COLLECTION_CONFIG_RULES,
            'Filter': {
                '_id': ObjectId(pid)
            }
        }

        mongodb = MongoAPI(data)
        response = mongodb.read()
    except Exception as e:
        log.exception(e)
        response = get_message_error(e)
    return response


def get_all_selected(cid: str, year: int, month: int, pid: str = ''):
    response = []
    try:
        data = {
            'collection': settings.MONGO_COLLECTION_PROC_RECONCILED_CNF,
            'Filter': {
                f'{settings.MONGO_COLLECTION_UPLOAD_FILE_PARTNER_ID}': cid,
                'year': int(year),
                'month': int(month)
            }
        }

        if len(pid) > 0:
            data['Filter']['_id'] = ObjectId(pid)

        mongodb = MongoAPI(data)
        config_proc = mongodb.read()
        if len(config_proc) > 0 and config_proc[0][f'{settings.MONGO_COLLECTION_CONFIG_RULES}_id'] != '':
            data = {
                'collection': settings.MONGO_COLLECTION_CONFIG_RULES,
                'Filter': {
                    f'{settings.MONGO_COLLECTION_UPLOAD_FILE_PARTNER_ID}': cid,
                    'config_state': 'terminate',
                    '_id': ObjectId(config_proc[0][f'{settings.MONGO_COLLECTION_CONFIG_RULES}_id'])
                }
            }

            mongodb = MongoAPI(data)
            rules = mongodb.read()
            for rule in rules:
                for cnf in config_proc:
                    response.append({
                        'name_rule': rule['name_rule'],
                        'rule_id': rule['_id'],
                        'created_at': cnf['created_at']
                    })
    except Exception as e:
        log.exception(e)
        response = get_message_error(e)
    return response


def del_all_selected(cid: str, year: int, month: int):
    response = []
    try:
        data = {
            'collection': settings.MONGO_COLLECTION_PROC_RECONCILED_CNF,
            'Filter': {
                f'{settings.MONGO_COLLECTION_UPLOAD_FILE_PARTNER_ID}': cid,
                'year': int(year),
                'month': int(month),
                'state': 'config'
            }
        }

        mongodb = MongoAPI(data)
        config_proc = mongodb.read()
        if config_proc:
            data = {
                'collection': settings.MONGO_COLLECTION_PROC_RECONCILED_CNF,
                'Filter': {
                    f'{settings.MONGO_COLLECTION_UPLOAD_FILE_PARTNER_ID}': cid,
                    'year': int(year),
                    'month': int(month),
                    '_id': ObjectId(config_proc['_id'])
                }
            }

            mongodb = MongoAPI(data)
            response = mongodb.delete(data)

    except Exception as e:
        log.exception(e)
        response = get_message_error(e)
    return response


def set_rule_selected(cid: str, json_data: dict):
    try:
        data = {
            'collection': settings.MONGO_COLLECTION_PROC_RECONCILED_CNF,
            'Document': {
                f'{settings.MONGO_COLLECTION_CONFIG_RULES}_id': json_data.get('config_id'),
                f'{settings.MONGO_COLLECTION_UPLOAD_FILE_PARTNER_ID}': cid,
                'user_id': json_data.get('user_id'),
                'year': int(json_data.get('year')),
                'month': int(json_data.get('month')),
                'created_at': get_datetime(),
                'state': constantes.CODE_STATE_CONFIG_PENDING
            },
            'Filter': {
                f'{settings.MONGO_COLLECTION_UPLOAD_FILE_PARTNER_ID}': cid,
                'year': int(json_data.get('year')),
                'month': int(json_data.get('month')),
                '_id': ObjectId(json_data.get('id'))
            },
            'DataToBeUpdated': {
                f'{settings.MONGO_COLLECTION_CONFIG_RULES}_id': json_data.get('config_id'),
                'updated_at': get_datetime()
            }
        }
        mongodb = MongoAPI(data)
        response = mongodb.write(data)
        if response and 'Document_ID' in response:
            response = detail_conf_reconciled(
                cid,
                int(json_data.get('year')),
                int(json_data.get('month')),
                response['Document_ID']
            )

    except Exception as e:
        log.exception(e)
        response = get_message_error(e)
    return response


def del_rule_selected(cid, json_data: dict):
    try:
        data = {
            'collection': settings.MONGO_COLLECTION_PROC_RECONCILED_CNF,
            'Filter': {
                '_id': ObjectId(json_data.get('id')),
                f'{settings.MONGO_COLLECTION_UPLOAD_FILE_PARTNER_ID}': cid,
                'year': int(json_data.get('year')),
                'month': int(json_data.get('month'))
            },
            'DataToBeUpdated': {
                f'{settings.MONGO_COLLECTION_CONFIG_RULES}_id': '',
                'updated_at': get_datetime()
            }
        }
        mongodb = MongoAPI(data)
        response = mongodb.update()
        if response and response['Status'] == 'Actualización exitosa':
            response = detail_conf_reconciled(
                cid,
                int(json_data.get('year')),
                int(json_data.get('month')),
                json_data.get('id')
            )

    except Exception as e:
        log.exception(e)
        response = get_message_error(e)
    return response


def new_config_reconciled(json_data: dict):
    try:
        data = {
            'collection': settings.MONGO_COLLECTION_PROC_RECONCILED_CNF,
            'Document': {
                f'{settings.MONGO_COLLECTION_UPLOAD_FILE_PARTNER_ID}': \
                    json_data.get(f'{settings.MONGO_COLLECTION_UPLOAD_FILE_PARTNER_ID}'),
                'year': json_data.get('year'),
                'month': json_data.get('month'),
                'status': constantes.CODE_STATE_CONFIG_PENDING,
                'created_at': get_datetime()
            },
            'Filter': {
                f'{settings.MONGO_COLLECTION_UPLOAD_FILE_PARTNER_ID}': \
                    json_data.get(f'{settings.MONGO_COLLECTION_UPLOAD_FILE_PARTNER_ID}'),
                'year': json_data.get('year'),
                'month': json_data.get('month'),
            }
        }
        mongodb = MongoAPI(data)
        response = mongodb.write(data)
    except Exception as e:
        log.exception(e)
        response = get_message_error(e)
    return response


def update_config_reconciled(json_data: dict):
    try:
        data = {
            'collection': settings.MONGO_COLLECTION_PROC_RECONCILED_CNF,
            'Filter': {
                '_id': ObjectId(json_data.get('id'))
            },
            'DataToBeUpdated': {
                'state': json_data.get('state'),
                'executed_count': 0,
                'updated_at': get_datetime()
            }
        }

        mongodb = MongoAPI(data)
        response = mongodb.write(data)
    except Exception as e:
        log.exception(e)
        response = get_message_error(e)
    return response


def get_detail_process(cid: str, year: int, month: int, pag: int, limit: int, report: str, type_process: str):
    offset = (int(limit) * int(pag)) - (int(limit) - 1)
    response = []
    process_conciliation = {}
    try:
        data = {
            'collection': settings.MONGO_COLLECTION_PROC_RECONCILED_CNF,
            'Filter': {
                f'{settings.MONGO_COLLECTION_UPLOAD_FILE_PARTNER_ID}': cid,
                'year': int(year),
                'month': int(month),
                'state': constantes.CODE_STATE_CONFIG_EXECUTED
            }
        }

        mongodb = MongoAPI(data)
        proc_recon = mongodb.read()
        for item in proc_recon:
            if report == constantes.OPTION_REPORTS_ALL or report == constantes.OPTION_REPORTS_RECONCILED:
                # Section Reconciled - Primary v/s External
                if type_process == constantes.OPTION_REPORTS_ALL or type_process == 'primary':
                    data = {
                        'collection': settings.MONGO_COLLECTION_DATA_RECONCILED,
                        'Filter': {
                            f'{settings.MONGO_COLLECTION_PROC_RECONCILED_CNF}_id': item['_id'],
                            'type_process': 'primary'
                        },
                        'Pagination': {
                            'offset': int(offset),
                            'limit': int(limit)
                        }
                    }
                    mongodb = MongoAPI(data)
                    reconciled = mongodb.read_with_pagination(data)

                    data = {
                        'collection': settings.MONGO_COLLECTION_DATA_TOTAL_RECONCILED,
                        'Filter': {
                            f'{settings.MONGO_COLLECTION_PROC_RECONCILED_CNF}_id': item['_id'],
                            'type_process': 'primary'
                        }
                    }
                    mongodb = MongoAPI(data)
                    reconciled_total = mongodb.read()

                    process_conciliation = {
                        'reconciled': {
                            'primary': {
                                'data': reconciled.get('data'),
                                'rows': reconciled.get('total'),
                                'limit': int(limit),
                                'act': int(pag),
                                'total': 0 if len(reconciled_total) == 0 else
                                int(reconciled_total[0]['total_reconciled'])
                            }
                        }
                    }

                # Section Reconciled - External v/s Primary
                if type_process == constantes.OPTION_REPORTS_ALL or type_process == 'secondary':
                    data = {
                        'collection': settings.MONGO_COLLECTION_DATA_RECONCILED,
                        'Filter': {
                            f'{settings.MONGO_COLLECTION_PROC_RECONCILED_CNF}_id': item['_id'],
                            'type_process': 'secondary'
                        },
                        'Pagination': {
                            'offset': int(offset),
                            'limit': int(limit)
                        }
                    }
                    mongodb = MongoAPI(data)
                    reconciled = mongodb.read_with_pagination(data)

                    data = {
                        'collection': settings.MONGO_COLLECTION_DATA_TOTAL_RECONCILED,
                        'Filter': {
                            f'{settings.MONGO_COLLECTION_PROC_RECONCILED_CNF}_id': item['_id'],
                            'type_process': 'secondary'
                        }
                    }
                    mongodb = MongoAPI(data)
                    reconciled_total = mongodb.read()

                    process_conciliation['reconciled']['secondary'] = {
                        'data': reconciled.get('data'),
                        'rows': reconciled.get('total'),
                        'limit': int(limit),
                        'act': int(pag),
                        'total': 0 if len(reconciled_total) == 0 else int(reconciled_total[0]['total_reconciled'])
                    }

            if report == constantes.OPTION_REPORTS_ALL or report == constantes.OPTION_REPORTS_UNRECONCILED:
                # Section Unreconciled - Primary v/s Secondary
                if type_process == constantes.OPTION_REPORTS_ALL or type_process == 'primary':
                    data = {
                        'collection': settings.MONGO_COLLECTION_DATA_UNRECONCILED,
                        'Filter': {
                            f'{settings.MONGO_COLLECTION_PROC_RECONCILED_CNF}_id': item['_id'],
                            'type_process': 'primary'
                        },
                        'Pagination': {
                            'offset': int(offset),
                            'limit': int(limit)
                        }
                    }
                    mongodb = MongoAPI(data)
                    unreconciled = mongodb.read_with_pagination(data)

                    data = {
                        'collection': settings.MONGO_COLLECTION_DATA_TOTAL_UNRECONCILED,
                        'Filter': {
                            f'{settings.MONGO_COLLECTION_PROC_RECONCILED_CNF}_id': item['_id'],
                            'type_process': 'primary'
                        },
                        'Pagination': {
                            'offset': int(offset),
                            'limit': int(limit)
                        }
                    }
                    mongodb = MongoAPI(data)
                    unreconciled_total = mongodb.read()

                    process_conciliation['unreconciled'] = {
                        'primary': {
                            'data': unreconciled.get('data'),
                            'rows': unreconciled.get('total'),
                            'limit': int(limit),
                            'act': int(pag),
                            'total': 0 if len(unreconciled_total) == 0 else
                            int(unreconciled_total[0]['total_unreconciled'])
                        }
                    }

                # Section Unreconciled - Secondary v/s Primary
                if type_process == constantes.OPTION_REPORTS_ALL or type_process == 'secondary':
                    data = {
                        'collection': settings.MONGO_COLLECTION_DATA_UNRECONCILED,
                        'Filter': {
                            f'{settings.MONGO_COLLECTION_PROC_RECONCILED_CNF}_id': item['_id'],
                            'type_process': 'secondary'
                        },
                        'Pagination': {
                            'offset': int(offset),
                            'limit': int(limit)
                        }
                    }
                    mongodb = MongoAPI(data)
                    unreconciled = mongodb.read_with_pagination(data)

                    data = {
                        'collection': settings.MONGO_COLLECTION_DATA_TOTAL_UNRECONCILED,
                        'Filter': {
                            f'{settings.MONGO_COLLECTION_PROC_RECONCILED_CNF}_id': item['_id'],
                            'type_process': 'secondary'
                        },
                        'Pagination': {
                            'offset': int(offset),
                            'limit': int(limit)
                        }
                    }
                    mongodb = MongoAPI(data)
                    unreconciled_total = mongodb.read()

                    process_conciliation['unreconciled']['secondary'] = {
                        'data': unreconciled.get('data'),
                        'rows': unreconciled.get('total'),
                        'limit': int(limit),
                        'act': int(pag),
                        'total': 0 if len(unreconciled_total) == 0 else int(unreconciled_total[0]['total_unreconciled'])
                    }

            if report == constantes.OPTION_REPORTS_ALL or report == constantes.OPTION_REPORTS_DUPLICATED:
                # Section Duplicated - Primary v/s Secondary
                if type_process == constantes.OPTION_REPORTS_ALL or type_process == 'primary':
                    data = {
                        'collection': settings.MONGO_COLLECTION_DATA_DUPLICATES,
                        'Filter': {
                            f'{settings.MONGO_COLLECTION_PROC_RECONCILED_CNF}_id': item['_id'],
                            'type_process': 'primary'
                        },
                        'Pagination': {
                            'offset': int(offset),
                            'limit': int(limit)
                        }
                    }
                    mongodb = MongoAPI(data)
                    duplicated = mongodb.read_with_pagination(data)

                    data = {
                        'collection': settings.MONGO_COLLECTION_DATA_TOTAL_DUPLICATES,
                        'Filter': {
                            f'{settings.MONGO_COLLECTION_PROC_RECONCILED_CNF}_id': item['_id'],
                            'type_process': 'primary'
                        }
                    }
                    mongodb = MongoAPI(data)
                    duplicated_total = mongodb.read()

                    process_conciliation['duplicated'] = {
                        'primary': {
                            'data': duplicated.get('data'),
                            'rows': duplicated.get('total'),
                            'limit': int(limit),
                            'act': int(pag),
                            'total': 0 if len(duplicated_total) == 0 else int(duplicated_total[0]['total_duplicated'])
                        }
                    }

                # Section Duplicated - Secondary v/s Primary
                if type_process == constantes.OPTION_REPORTS_ALL or type_process == 'secondary':
                    data = {
                        'collection': settings.MONGO_COLLECTION_DATA_DUPLICATES,
                        'Filter': {
                            f'{settings.MONGO_COLLECTION_PROC_RECONCILED_CNF}_id': item['_id'],
                            'type_process': 'secondary'
                        },
                        'Pagination': {
                            'offset': int(offset),
                            'limit': int(limit)
                        }
                    }
                    mongodb = MongoAPI(data)
                    duplicated = mongodb.read_with_pagination(data)

                    data = {
                        'collection': settings.MONGO_COLLECTION_DATA_TOTAL_DUPLICATES,
                        'Filter': {
                            f'{settings.MONGO_COLLECTION_PROC_RECONCILED_CNF}_id': item['_id'],
                            'type_process': 'secondary'
                        }
                    }
                    mongodb = MongoAPI(data)
                    duplicated_total = mongodb.read()

                    process_conciliation['duplicated']['secondary'] = {
                        'data': duplicated.get('data'),
                        'rows': duplicated.get('total'),
                        'limit': int(limit),
                        'act': int(pag),
                        'total': 0 if len(duplicated_total) == 0 else int(duplicated_total[0]['total_duplicated'])
                    }
            # Se genera mensaje de respuesta
            response = process_conciliation
    except Exception as e:
        log.exception(e)
        response = get_message_error(e)
    return response


def get_rule_not_selected(cid: str, json_data: dict):
    return detail_conf_reconciled(cid, int(json_data.get('year')), int(json_data.get('month')), json_data.get('id'))


def detail_conf_reconciled(cid: str, year: int, month: int, pid: str = ''):
    response = {}
    data = {
        'collection': settings.MONGO_COLLECTION_PROC_RECONCILED_CNF,
        'Filter': {
            '_id': ObjectId(pid),
            f'{settings.MONGO_COLLECTION_UPLOAD_FILE_PARTNER_ID}': cid,
            'year': int(year),
            'month': int(month)
        }
    }
    mongodb = MongoAPI(data)
    cnf_reconciled = mongodb.read()
    if len(cnf_reconciled) > 0:
        for cnf in cnf_reconciled:
            response = {
                f'{settings.MONGO_COLLECTION_UPLOAD_FILE_PARTNER_ID}': cid,
                'year': int(year),
                'month': int(month),
                'config_id': cnf[f'{settings.MONGO_COLLECTION_CONFIG_RULES}_id'],
                'user_id': cnf['user_id'],
                'state': cnf['state'],
                'id': cnf['_id'],
                'rules': get_all_config_rules_by_partner(cid, int(year), int(month), pid)
            }

            if cnf[f'{settings.MONGO_COLLECTION_CONFIG_RULES}_id'] != '':
                response['ruleSelected'] = get_all_selected(cid, int(year), int(month), pid)
            else:
                response['ruleSelected'] = []
    else:
        response = {}

    return response


def download_report_by_type(cid: str, year: int, month: int, report: str, type_process: str):
    data_report = []
    data = {
        'collection': settings.MONGO_COLLECTION_PROC_RECONCILED_CNF,
        'Filter': {
            f'{settings.MONGO_COLLECTION_UPLOAD_FILE_PARTNER_ID}': cid,
            'year': int(year),
            'month': int(month)
        }
    }
    mongodb = MongoAPI(data)
    cnf_processes = mongodb.read()
    for cnf_process in cnf_processes:
        if report == constantes.OPTION_REPORTS_RECONCILED:
            # Section Reconciled
            data = {
                'collection': settings.MONGO_COLLECTION_DATA_RECONCILED,
                'Filter': {
                    f'{settings.MONGO_COLLECTION_PROC_RECONCILED_CNF}_id': cnf_process['_id'],
                    'type_process': type_process
                }
            }
            mongodb = MongoAPI(data)
            data_report = mongodb.read()

        if report == constantes.OPTION_REPORTS_ALL or report == constantes.OPTION_REPORTS_UNRECONCILED:
            # Section Unreconciled
            data = {
                'collection': settings.MONGO_COLLECTION_DATA_UNRECONCILED,
                'Filter': {
                    f'{settings.MONGO_COLLECTION_PROC_RECONCILED_CNF}_id': cnf_process['_id'],
                    'type_process': type_process
                }
            }
            mongodb = MongoAPI(data)
            data_report = mongodb.read()

        if report == constantes.OPTION_REPORTS_ALL or report == constantes.OPTION_REPORTS_DUPLICATED:
            # Section Duplicated
            data = {
                'collection': settings.MONGO_COLLECTION_DATA_DUPLICATES,
                'Filter': {
                    f'{settings.MONGO_COLLECTION_PROC_RECONCILED_CNF}_id': cnf_process['_id'],
                    'type_process': type_process
                }
            }
            mongodb = MongoAPI(data)
            data_report = mongodb.read()

        if len(data_report) > 0:
            docs = pd.DataFrame(list(data_report))
            return docs


#####################################
#    Definición Métodos Privados
#####################################
def __create_upload_file(
        time_process: str,
        filename: str,
        ext: str,
        type_data: str,
        year: int,
        month: int):
    """
    Carga Configuración Archivos
    :param time_process:
    :param filename:
    :param ext:
    :param type_data:
    :param year:
    :param month:
    :return:
    """
    data = {
        'collection': settings.MONGO_COLLECTION_UPLOAD_FILE,
        'Document': {
            'year': int(year),
            'month': int(month),
            'filename': filename,
            'extension': ext,
            'file_processed': False,
            'type': type_data,
            'created_at': time_process
        },
        'Filter': {
            'year': int(year),
            'month': int(month),
            'filename': filename,
            'extension': ext
        },
        'DataToBeUpdated': {
            'file_processed': False,
            'date_upload': time_process,
            'updated_at': time_process
        }
    }

    mongodb = MongoAPI(data)
    return mongodb.write(data)


def __get_state_config_reconciled(state: str):
    if state == 'config':
        return constantes.STATE_CONFIG_RECONCILED
    elif state == 'pending':
        return constantes.STATE_CONFIG_PENDING
    elif state == 'executed':
        return constantes.STATE_CONFIG_EXECUTED
    else:
        return constantes.STATE_CONFIG_IN_PROCESS


def __eliminar_campo(diccionarios, campo):
    if not diccionarios:  # Verificar si la lista está vacía
        return []

    return list(map(lambda d: {key: value for key, value in d.items() if key != campo}, diccionarios))



import logging

from bson import ObjectId

from ..config import settings
from ..database.mongodb import MongoAPI
from ..utility.utils import get_datetime, get_message_error

log = logging.getLogger(__name__)


def get_all_partners(pid: str):
    try:
        data = {
            'collection': settings.MONGO_COLLECTION_DATA_PARTNERS,
            'Filter': {
                'active': True,
                'user_id': pid
            }
        }
        mongodb = MongoAPI(data)
        response = mongodb.all()
    except Exception as e:
        log.exception(e)
        response = get_message_error(e)
    return response


def new_partner(request):
    try:
        if 'id' in request:
            data = {
                'collection': settings.MONGO_COLLECTION_DATA_PARTNERS,
                'Filter': {
                    '_id': ObjectId(request.get('id')),
                    'user_id': request.get('user_id')
                },
                'DataToBeUpdated': {
                    'name': request.get('name'),
                    'updated_at': get_datetime(),
                }
            }
            mongodb = MongoAPI(data)
            response = mongodb.update()
        else:
            data = {
                'collection': settings.MONGO_COLLECTION_DATA_PARTNERS,
                'Document': {
                    'name': request.get('name'),
                    'user_id': request.get('user_id'),
                    'active': request.get('active'),
                    'created_at': get_datetime(),
                },
                'Filter': {
                    'name': request.get('name')
                }
            }
            mongodb = MongoAPI(data)
            response = mongodb.write(data, True)
    except Exception as e:
        log.exception(e)
        response = get_message_error(e)

    return response


def get_partner(pid: str):
    try:
        data = {
            'collection': settings.MONGO_COLLECTION_DATA_PARTNERS,
            'Filter': {
                '_id': ObjectId(pid)
            }
        }
        mongodb = MongoAPI(data)
        response = mongodb.read()
    except Exception as e:
        log.exception(e)
        response = get_message_error(e)
    return response


def update_partner(pid: str, request):
    try:
        data = {
            'collection': settings.MONGO_COLLECTION_DATA_PARTNERS,
            'Filter': {
                '_id': ObjectId(pid)
            },
            'DataToBeUpdated': {
                'name': request.get('nombre'),
                'active': request.get('activo'),
                'updated_at': get_datetime(),
            }
        }
        mongodb = MongoAPI(data)
        response = mongodb.update()
    except Exception as e:
        log.exception(e)
        response = get_message_error(e)
    return response


def delete_partner(pid: str):
    try:
        data = {
            'collection': settings.MONGO_COLLECTION_DATA_PARTNERS,
            'Filter': {
                '_id': ObjectId(pid)
            }
        }
        mongodb = MongoAPI(data)
        response = mongodb.write(data)

        if response:
            data = {
                'collection': settings.MONGO_COLLECTION_DATA_PARTNERS,
                'Filter': {
                    '_id': ObjectId(pid)
                },
                'DataToBeUpdated': {
                    'active': False,
                    'updated_at': get_datetime(),
                }
            }
            mongodb = MongoAPI(data)
            response = mongodb.update()
            if response:
                return {'Status': 'Eliminación exitosa'}

    except Exception as e:
        log.exception(e)
        response = get_message_error(e)
    return response



from .utils import (
    empty_result,
    set_default,
    cast_id_mongo,
    get_uuid,
    get_datetime,
    allowed_file,
    password,
    check_password,
    get_mime_type_application,
    get_message_error,
    get_start_end_day_month,
    serializer,
    extract_data_line,
    DateTimeEncoder
)
from .utils_file import (
    extract_headers_file,
    create_file_from_dataframe,
    execute_process_conciliation,
    convert_file_to_hfs5
)
from .utils_messages import UtilsMessages



from .conciliation import (
    upload_file,
    upload_file_by_partner,
    get_files_upload,
    get_index_file,
    get_columns_amount,
    get_columns_date,
    process_files,
    get_all,
    get_data_rule,
    get_all_selected,
    del_all_selected,
    set_rule_selected,
    del_rule_selected,
    new_config_reconciled,
    update_config_reconciled,
    get_detail_process,
    get_rule_not_selected,
    detail_conf_reconciled,
    download_report_by_type
)
from .config_rules import (
    create_config_rule,
    update_config_rule,
    validate_file,
    get_files,
    get_file,
    get_columns_file,
    set_columns_file,
    create_index_file,
    create_amount_column,
    create_date_column,
    set_select_files_to_process,
    get_index_file,
    get_all_process_pending,
    get_all,
    set_mapping_columns,
    get_mapping_columns,
    get_all_config_rules_by_partner,
    config_remove
)
from .cycles import (
    new_cycle,
    get_cycle,
    update_cycle,
    delete_cycle,
    get_all_cycles
)
from .partner import (
    new_partner,
    get_partner,
    update_partner,
    delete_partner,
    get_all_partners
)
from .role import (
    new_role,
    get_role,
    update_role,
    delete_role,
    get_all_roles,
    check_role_exists
)
from .type_cycles import (
    new_type_cycle,
    get_type_cycle,
    update_type_cycle,
    delete_type_cycle,
    get_all_type_cycles
)
from .users import (
    new_user,
    get_user,
    get_all_users,
    update_user,
    delete_user,
    get_all_users_by_rol,
    check_user_exists,
    check_current_user
)
from .messages import ( petition )



from enum import Enum, unique


@unique
class DataTypeEnum(Enum):
    STRING = 'str'
    DATE = 'datetime'
    INTEGER = 'int'
    FLOAT = 'float'


@unique
class ReconciledStateEnum(Enum):
    RECONCILED_INIT = 'launched'
    RECONCILED_IN_PROCESS = 'in-process'
    RECONCILED_TERMINATE_OK = 'terminate-success'
    RECONCILED_TERMINATE_NOK = 'terminate-error'



import oracledb
from sqlalchemy import create_engine, NullPool
from sqlalchemy.orm import sessionmaker

from ..config import settings


class OracleAPI:
    def __init__(self):
        self.pool = oracledb.create_pool(
            user=settings.ORACLE_USERNAME,
            password=settings.ORACLE_PASSWORD,
            host=settings.ORACLE_HOST,
            port=settings.ORACLE_PORT,
            service_name=settings.ORACLE_SERVICE_NAME,
            min=100,
            max=100,
            increment=0
        )

        self.engine = create_engine("oracle+oracledb://", creator=self.pool.acquire, poolclass=NullPool)

        # Create a session to the database
        self.session = sessionmaker(bind=self.engine)()

    def read(self, sql):
        return self.session.execute(sql).fetchone()

    def all(self, sql):
        return self.session.execute(sql).fetchall()



import logging

from bson import ObjectId

from ..config import settings
from ..database.mongodb import MongoAPI
from ..utility.utils import get_datetime, get_message_error

log = logging.getLogger(__name__)


def new_type_cycle(json: dict):
    try:
        if 'id' in json:
            data = {
                'collection': settings.MONGO_COLLECTION_DATA_TYPE_CYCLES,
                'Filter': {
                    '_id': ObjectId(json.get('id')),
                    'active': True
                },
                'DataToBeUpdated': {
                    'name_type_cycle': json.get('name_type_cycle'),
                    'updated_at': get_datetime(),
                }
            }
            mongodb = MongoAPI(data)
            response = mongodb.update()
        else:
            data = {
                'collection': settings.MONGO_COLLECTION_DATA_TYPE_CYCLES,
                'Document': {
                    'name_type_cycle': json.get('name_type_cycle'),
                    'active': True,
                    'created_at': get_datetime(),
                },
                'Filter': {
                    'name_type_cycle': json.get('name_type_cycle')
                }
            }
            mongodb = MongoAPI(data)
            response = mongodb.write(data)
    except Exception as e:
        log.exception(e)
        response = get_message_error(e)

    return response


def get_type_cycle(pid: str):
    try:
        data = {
            'collection': settings.MONGO_COLLECTION_DATA_TYPE_CYCLES,
            'Filter': {
                '_id': ObjectId(pid),
                'active': True,
            }
        }
        mongodb = MongoAPI(data)
        response = mongodb.read()
    except Exception as e:
        log.exception(e)
        response = get_message_error(e)

    return response


def update_type_cycle(pid: str, json):
    try:
        data = {
            'collection': settings.MONGO_COLLECTION_DATA_TYPE_CYCLES,
            'Filter': {
                '_id': ObjectId(pid)
            },
            'DataToBeUpdated': {
                'name_type_cycle': json.get('name_type_cycle'),
                'updated_at': get_datetime(),
            }
        }
        mongodb = MongoAPI(data)
        response = mongodb.update()
    except Exception as e:
        log.exception(e)
        response = get_message_error(e)

    return response


def delete_type_cycle(pid: str):
    try:
        data = {
            'collection': settings.MONGO_COLLECTION_DATA_TYPE_CYCLES,
            'Filter': {
                '_id': ObjectId(pid)
            }
        }
        mongodb = MongoAPI(data)
        response = mongodb.write(data)

        if response:
            data = {
                'collection': settings.MONGO_COLLECTION_DATA_TYPE_CYCLES,
                'Filter': {
                    '_id': ObjectId(pid)
                },
                'DataToBeUpdated': {
                    'active': False,
                    'updated_at': get_datetime(),
                }
            }
            mongodb = MongoAPI(data)
            response = mongodb.update()
            if response:
                response = {'Status': 'Eliminación exitosa'}

    except Exception as e:
        log.exception(e)
        response = get_message_error(e)

    return response


def get_all_type_cycles():
    try:
        data = {
            'collection': settings.MONGO_COLLECTION_DATA_TYPE_CYCLES,
            'Filter': {
                'active': True
            }
        }
        mongodb = MongoAPI(data)
        response = mongodb.all()
    except Exception as e:
        log.exception(e)
        response = get_message_error(e)
    return response



# Version liberación - Release ddmmyyyy
APP_VERSION = '1.6.0'
# Flask Settings
FLASK_SECRET_KEY = '0pt1m1$42560-$3cr3t4'
FLASK_SERVER_NAME = 'localhost' # '0.0.0.0'  # '20.206.88.85'
FLASK_SERVER_PORT = 9090  # 5000
FLASK_DEBUG = True
FLASK_THREADED = True
FLASK_ERROR_404_HELP = False
FLASK_APP_FOLDER = 'optidata-core'
FLASK_NAME_UPLOAD_FOLDER = 'files'
FLASK_NAME_EXPORT_FOLDER = 'exported-files'
FLASK_ALLOWED_EXTENSIONS = {'txt', 'csv', 'xls', 'xlsx'}
FLASK_MAX_CONTENT_LENGTH = 500 * 1024 * 1024
FLASK_NUMBER_PROCESSES = 2
FLASK_ROOT_PATH = ''
FLASK_UPLOAD_FOLDER = ''
FLASK_EXPORT_FOLDER = ''

# Flask-Restplus settings
RESTPLUS_SWAGGER_UI_DOC_EXPANSION = 'list'
RESTPLUS_VALIDATE = True
RESTPLUS_MASK_SWAGGER = False
RESTPLUS_ERROR_404_HELP = False
RESTPLUS_API_VERSION = '/v1'
RESTPLUS_EMPTY_RESULT = 'No existen registros asociados'

# Oracle Properties (SAP)
ORACLE_HOST = 'localhost'
ORACLE_PORT = '1521'
ORACLE_USERNAME = 'sys as sysadmin'
ORACLE_PASSWORD = 'Opt1m1$42560'
ORACLE_DB_NAME = 'orcl'
ORACLE_SERVICE_NAME = ''
ORACLE_DNS = f'{ORACLE_HOST}/{ORACLE_DB_NAME}'
ORACLE_ENCODING = 'UTF-8'

# MongoDB Configuration
MONGO_BATCH_SIZE = 100000
MONGO_DB_HOST = 'localhost'  # 'cxp-db'  # '20.206.88.85'  #'cxpnew2.brazilsouth.cloudapp.azure.com'  # 'cxp-db'
MONGO_DB_PORT = 27017
MONGO_DB_NAME = 'cxp_db'
MONGO_DB_URI = f'mongodb://{MONGO_DB_HOST}:{MONGO_DB_PORT}/{MONGO_DB_NAME}'
MONGO_DB_USER = ''
MONGO_DB_PWD = ''
MONGO_COLLECTIONS = {
    'reconciled',
    'total_reconciled',
    'unreconciled',
    'total_unreconciled',
    'duplicates',
    'total_duplicates',
    'config_rules',
    'config_rules_columns',
    'config_rules_columns_index',
    'config_rules_column_amount',
    'config_rules_column_date',
    'config_rules_columns_mapping',
    'upload_files',
    'config_process',
    'config_process_files',
    'users',
    'roles',
    'partners',
    'type_cycles',
    'cycles',
    'sersal',
    'sersal_detail',
    'cartola',
    'cartola_detail',
    'program_scheduler'
}

# MongoDB Documents
MONGO_COLLECTION_UPLOAD_FILE_USER_ID = 'user_id'
MONGO_COLLECTION_UPLOAD_FILE_PARTNER_ID = 'partner_id'
MONGO_COLLECTION_UPLOAD_FILE_CLIENT_ID = 'partner_id'
MONGO_COLLECTION_UPLOAD_FILE = 'upload_files'
MONGO_COLLECTION_CONFIG_RULES = 'config_rules'
MONGO_COLLECTION_CONFIG_RULES_DEFINED_COLUMNS = 'config_rules_columns'
MONGO_COLLECTION_CONFIG_RULES_DEFINED_INDEX_COLUMNS = 'config_rules_columns_index'
MONGO_COLLECTION_CONFIG_RULES_DEFINED_AMOUNT_COLUMN = 'config_rules_column_amount'
MONGO_COLLECTION_CONFIG_RULES_DEFINED_DATE_COLUMN = 'config_rules_column_date'
MONGO_COLLECTION_CONFIG_RULES_DEFINED_MAPPING_COLUMNS = 'config_rules_columns_mapping'
MONGO_COLLECTION_PROC_RECONCILED_CNF = 'config_process'
MONGO_COLLECTION_PROC_RECONCILED_FILE = 'config_process_files'
MONGO_COLLECTION_DATA_RECONCILED = 'reconciled'
MONGO_COLLECTION_DATA_TOTAL_RECONCILED = 'total_reconciled'
MONGO_COLLECTION_DATA_DUPLICATES = 'duplicates'
MONGO_COLLECTION_DATA_TOTAL_DUPLICATES = 'total_duplicates'
MONGO_COLLECTION_DATA_UNRECONCILED = 'unreconciled'
MONGO_COLLECTION_DATA_TOTAL_UNRECONCILED = 'total_unreconciled'
MONGO_COLLECTION_DATA_USERS = 'users'
MONGO_COLLECTION_DATA_ROLES = 'roles'
MONGO_COLLECTION_DATA_PARTNERS = 'partners'
MONGO_COLLECTION_DATA_TYPE_CYCLES = 'type_cycles'
MONGO_COLLECTION_DATA_CYCLES = 'cycles'
MONGO_COLLECTION_DATA_SERSAL_CAB = 'sersal'
MONGO_COLLECTION_DATA_SERSAL_DET = 'sersal_detail'
MONGO_COLLECTION_DATA_CARTOLA_CAB = 'cartola'
MONGO_COLLECTION_DATA_CARTOLA_DET = 'cartola_detail'
MONGO_COLLECTION_DATA_PROG_SCHEDULERS = 'program_scheduler'

# BCrypt Config
BCRYPT_LOG_ROUNDS = 13

# JWT Configuration
TOKEN_EXPIRE_HOURS = 1
TOKEN_EXPIRE_MINUTES = 15
TOKEN_REFRESH_EXPIRE_HOURS = 1
TOKEN_HEADER_NAME = 'X-Access-Token'
TOKEN_FORMAT = 'Bearer '

# Files Configuration
FILE_EXT_TXT = 'txt'
FILE_EXT_CSV = 'csv'
FILE_EXT_XLS = 'xls'
FILE_EXT_XLSX = 'xlsx'
CSV_SEPARATOR = ';'

# Vaex Configuration
VAEX_THREAD_COUNT = 10
VAEX_MAX_COLUMN = 10000

# Default Configurations - User/Role
DEFAULT_USERNAME = 'admin@optimisa.cl'
DEFAULT_PASSWORD = 123456
DEFAULT_FIRSTNAME = 'Administrador'
DEFAULT_LASTNAME = 'del Sistema'

DEFAULT_USER_USERNAME = 'consulta@optimisa.cl'
DEFAULT_USER_PASSWORD = 123456
DEFAULT_USER_FIRSTNAME = 'Usuario Consulta'
DEFAULT_USER_LASTNAME = 'del Sistema'
DEFAULT_ROLE_NAME_ADMIN = 'Administrador'
DEFAULT_ROLE_NAME_USER = 'Consulta'

# Kafka Properties
KAFKA_BOOSTRAP = 'localhost'
KAFKA_PORT = 19092
KAFKA_SASL_MECHANISM = 'PLAIN'
KAFKA_SECURITY_PROTOCOL = 'SASL_SSL'
KAFKA_USERNAME = ''
KAFKA_PASSWORD = ''
KAFKA_OFFSET = 'earliest'
KAFKA_TIMEOUT = 1000
KAFKA_ENCODE = 'UTF-8'

# Kafka Topics
KAFKA_TOPIC_SCHEDULER = ''
KAFKA_TOPIC_PETITION = ''

# SFTP Properties
SFTP_HOSTNAME = 'localhost'
SFTP_USERNAME = 'optimisa'
SFTP_PASSWORD = '0pt1m1$42560'
SFTP_PORT = 2323
SFTP_PATH_DEFAULT = 'sftp'

SFTP_PATH_CARTOLAS = 'cartolas'
SFTP_PATH_LOTES = 'lotes_pago'
SFTP_PATH_SERSAL = 'sersal'

SFTP_SERSAL_FILE_FORMAT = 'SERSALRYYYYMMDD'

# Scheduler Properties
SCHEDULER_MIN_TIME_RUN = 1
SCHEDULER_MIN_TIME_RUN_LOTES = 1
SCHEDULER_MIN_TIME_RUN_CARTOLA = 1
SCHEDULER_MIN_TIME_RUN_PA = 1



import datetime
import logging

from bson import ObjectId

from ..config import settings
from ..database.mongodb import MongoAPI
from ..services.role import get_role
from ..utility.utils import password, get_message_error, get_datetime

log = logging.getLogger(__name__)


def new_user(data):
    try:
        date_doc = datetime.datetime.now()
        data = {
            'collection': settings.MONGO_COLLECTION_DATA_USERS,
            'Document': {
                'username': data.get('username'),
                'password': password(data.get('password')),
                'firstname': data.get('nombre'),
                'lastname': data.get('apellidos'),
                'rol_id': data.get('rol_id'),
                'active': data.get('active'),
                'created_at': date_doc
            },
            'Filter': {
                'username': data.get('username')
            },
            'DataToBeUpdated': {
                'firstname': data.get('nombre'),
                'lastname': data.get('apellidos'),
                'rol_id': data.get('rol_id'),
                'updated_at': date_doc,
            }
        }

        mongodb = MongoAPI(data)
        response = mongodb.write(data)

    except Exception as e:
        log.exception(e)
        response = get_message_error(e)
    return response


def get_user(pid: str):
    try:
        data = {
            'collection': settings.MONGO_COLLECTION_DATA_USERS,
            'Filter': {
                '_id': ObjectId(pid)
            }
        }
        mongodb = MongoAPI(data)
        response = mongodb.read()
    except Exception as e:
        log.exception(e)
        response = get_message_error(e)
    return response


def get_all_users():
    users = []
    try:
        data = {
            'collection': settings.MONGO_COLLECTION_DATA_USERS,
            'Filter': {
                'active': True
            }
        }
        mongodb = MongoAPI(data)
        response = mongodb.all()
        if response:
            for user in response:
                rol = get_role(user['rol_id'])
                if rol:
                    users.append({
                        'id': user['_id'],
                        'username': user['username'],
                        'firstname': user['firstname'],
                        'lastname': user['lastname'],
                        'rol_name': rol[0]['description'],
                        'rol_id': user['rol_id'],
                        'active': user['active'],
                    })
    except Exception as e:
        log.exception(e)
        users = get_message_error(e)
    return users


def update_user(pid: str, request):
    try:
        data = {
            'collection': settings.MONGO_COLLECTION_DATA_PARTNERS,
            'Filter': {
                '_id': ObjectId(pid)
            },
            'DataToBeUpdated': {
                'firstname': request.get('nombre'),
                'lastname': request.get('apellidos'),
                'active': request.get('activo'),
                'updated_at': get_datetime(),
            }
        }
        mongodb = MongoAPI(data)
        response = mongodb.update()
    except Exception as e:
        log.exception(e)
        response = get_message_error(e)
    return response


def delete_user(pid: str):
    try:
        data = {
            'collection': settings.MONGO_COLLECTION_DATA_USERS,
            'Filter': {
                '_id': ObjectId(pid)
            }
        }
        mongodb = MongoAPI(data)
        response = mongodb.write(data)

        if response:
            data = {
                'collection': settings.MONGO_COLLECTION_DATA_USERS,
                'Filter': {
                    '_id': ObjectId(pid)
                },
                'DataToBeUpdated': {
                    'active': False,
                    'updated_at': get_datetime(),
                }
            }
            mongodb = MongoAPI(data)
            response = mongodb.update()
            if response:
                return {'Status': 'Eliminación exitosa'}

    except Exception as e:
        log.exception(e)
        response = get_message_error(e)
    return response


def get_all_users_by_rol(rol_id):
    try:
        data = {
            'collection': settings.MONGO_COLLECTION_DATA_USERS,
            'Filter': {
                'active': True,
                'rol_id': rol_id
            }
        }
        mongodb = MongoAPI(data)
        response = mongodb.all()
    except Exception as e:
        log.exception(e)
        response = get_message_error(e)
    return response


def check_user_exists(description):
    try:
        data = {
            'collection': settings.MONGO_COLLECTION_DATA_USERS,
            'Filter': {
                'username': description
            }
        }
        mongodb = MongoAPI(data)
        response = mongodb.read()
    except Exception as e:
        log.exception(e)
        response = get_message_error(e)
    return response


def check_current_user(pid):
    try:
        data = {
            'collection': settings.MONGO_COLLECTION_DATA_USERS,
            'Filter': {
                '_id': ObjectId(pid)
            }
        }
        mongodb = MongoAPI(data)
        response = mongodb.read()
    except Exception as e:
        log.exception(e)
        response = get_message_error(e)
    return response



from setuptools import setup, find_packages

# from app.optidata.core.src.config import settings

VERSION = '1.6.0' # settings.APP_VERSION
DESCRIPTION = 'Paquete de Python para OptiData'
LONG_DESCRIPTION = 'Paquete para OptiData que contiene funcionalidades para realizar una Conciliación con Pandas y Vaex'

# Configurando
setup(
    name="optidata-core",
    version=VERSION,
    author="Gonzalo Torres Moya",
    author_email="<gtorres@optimisa.cl>",
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    packages=find_packages(where="app"),
    package_dir={"": "app"},
    keywords=['python', 'optidata-core'],
    license="MIT",
    install_requires=[
        "Flask-Bcrypt==1.0.1",
        "Flask>=3.0.2",
        "numpy>=1.26.4",
        "openpyxl>=3.1.2",
        "pandas>=2.2.1",
        "pymongo==4.3.3",
        "pysftp==0.2.9",
        "vaex>=4.17.0",
        "kafka-python>=2.0.2",
        "joblib>=1.3.2",
        "paramiko~=3.4.0",
        "oracledb==2.1.0",
        "sqlalchemy==2.0.28",
        "APScheduler>=3.10.4",
        "flask-restx>=1.3.0"
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.9"
)



import logging
from flask import make_response
from flask_restx import Api

from ..config import settings
from ..config.error_handling import AppErrorBaseClass, ObjectNotFound

log = logging.getLogger(__name__)
authorizations = {
    'Bearer': {
        'type': 'apiKey',
        'in': 'header',
        'name': 'X-Access-Token',
        'description': "Type in the *'Value'* input box below: **'Bearer &lt;JWT&gt;'**, where JWT is the token"
    }
}
api = Api(
    authorizations=authorizations,
    security='Bearer',
    title='API Motor Conciliación',
    version=f'1.0 - Release({settings.APP_VERSION})',
    description='API Backend para Motor Conciliación con JWT-Based Authentication',
    prefix=settings.RESTPLUS_API_VERSION,
    license='MIT',
    contact='Gonzalo Ariel Torres Moya',
    contact_url='mailto:gtorres@optimisa.cl',
    contact_email='gtorres@optimisa.cl',
    ordered=False,
    catch_all_404s=True,
    default_mediatype='application/json'
)


@api.errorhandler
def default_error_handler():
    message = 'An unhandled exception occurred.'
    log.exception(message)
    if not settings.FLASK_DEBUG:
        return make_response({'message': message}, 500)


@api.errorhandler(AppErrorBaseClass)
def handle_app_base_error(e):
    return make_response({'msg': str(e)}, 500)


@api.errorhandler(ObjectNotFound)
def handle_object_not_found_error(e):
    return make_response({'msg': str(e)}, 404)

